@@ -6,7 +6,7 @@ from collections.abc import AsyncIterator, Mapping, Sequence
 from contextlib import AbstractAsyncContextManager, asynccontextmanager
 from datetime import datetime
 from decimal import Decimal
-from typing import Any, overload
+from typing import Any, Iterable, List, Tuple, Union, overload
 
 import aiohttp
 from multidict import CIMultiDict
@@ -24,6 +24,10 @@ from neuro_admin_client.entities import (
     OrgUser,
     OrgUserRoleType,
     OrgUserWithInfo,
+    Project,
+    ProjectUser,
+    ProjectUserRoleType,
+    ProjectUserWithInfo,
     Quota,
     User,
     UserInfo,
@@ -34,6 +38,14 @@ def _to_query_bool(flag: bool) -> str:
     return str(flag).lower()
 
 
+GetUserRet = Union[
+    User,
+    Tuple[User, List[ClusterUser]],
+    Tuple[User, List[ProjectUser]],
+    Tuple[User, List[ClusterUser], List[ProjectUser]],
+]
+
+
 class AdminClientABC(abc.ABC):
     async def __aenter__(self) -> "AdminClientABC":
         return self
@@ -48,10 +60,42 @@ class AdminClientABC(abc.ABC):
     async def list_users(self) -> list[User]:
         ...
 
-    @abstractmethod
+    @overload
     async def get_user(self, name: str) -> User:
         ...
 
+    @overload
+    async def get_user(
+        self, name: str, *, include_clusters: Literal[True]
+    ) -> tuple[User, list[ClusterUser]]:
+        ...
+
+    @overload
+    async def get_user(
+        self, name: str, *, include_projects: Literal[True]
+    ) -> tuple[User, list[ProjectUser]]:
+        ...
+
+    @overload
+    async def get_user(
+        self,
+        name: str,
+        *,
+        include_clusters: Literal[True],
+        include_projects: Literal[True],
+    ) -> tuple[User, list[ClusterUser], list[ProjectUser]]:
+        ...
+
+    @abstractmethod
+    async def get_user(
+        self,
+        name: str,
+        *,
+        include_clusters: bool = False,
+        include_projects: bool = False,
+    ) -> GetUserRet:
+        ...
+
     @abstractmethod
     async def get_user_with_clusters(self, name: str) -> tuple[User, list[ClusterUser]]:
         ...
@@ -63,6 +107,7 @@ class AdminClientABC(abc.ABC):
         email: str,
         first_name: str | None = None,
         last_name: str | None = None,
+        skip_auto_add_to_clusters: bool = False,
     ) -> User:
         ...
 
@@ -524,6 +569,7 @@ class AdminClientABC(abc.ABC):
     async def create_org(
         self,
         name: str,
+        skip_auto_add_to_clusters: bool = False,
     ) -> Org:
         ...
 
@@ -621,6 +667,165 @@ class AdminClientABC(abc.ABC):
     async def delete_org_user(self, org_name: str, user_name: str) -> None:
         ...
 
+    # projects
+
+    @abstractmethod
+    async def create_project(
+        self,
+        name: str,
+        cluster_name: str,
+        org_name: str | None,
+        is_default: bool = False,
+        default_role: ProjectUserRoleType = ProjectUserRoleType.WRITER,
+    ) -> Project:
+        ...
+
+    @abstractmethod
+    async def list_projects(
+        self, cluster_name: str, org_name: str | None = None
+    ) -> list[Project]:
+        ...
+
+    @abstractmethod
+    async def get_project(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+    ) -> Project:
+        ...
+
+    @abstractmethod
+    async def update_project(self, project: Project) -> None:
+        ...
+
+    @abstractmethod
+    async def delete_project(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+    ) -> None:
+        ...
+
+    #  project user
+
+    @overload
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: Literal[True],
+    ) -> list[ProjectUserWithInfo]:
+        ...
+
+    @overload
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: Literal[False] = ...,
+    ) -> list[ProjectUser]:
+        ...
+
+    @abstractmethod
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: bool = False,
+    ) -> list[ProjectUser] | list[ProjectUserWithInfo]:
+        ...
+
+    @overload
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: Literal[True],
+    ) -> ProjectUserWithInfo:
+        ...
+
+    @overload
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: Literal[False] = ...,
+    ) -> ProjectUser:
+        ...
+
+    @abstractmethod
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: bool = False,
+    ) -> ProjectUser | ProjectUserWithInfo:
+        ...
+
+    @overload
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: Literal[True],
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUserWithInfo:
+        ...
+
+    @overload
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: Literal[False] = ...,
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUser:
+        ...
+
+    @abstractmethod
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: bool = False,
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUser | ProjectUserWithInfo:
+        ...
+
+    @abstractmethod
+    async def update_project_user(self, project_user: ProjectUser) -> None:
+        ...
+
+    @abstractmethod
+    async def delete_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+    ) -> None:
+        ...
+
     # OLD API:
 
     @abstractmethod
@@ -641,7 +846,7 @@ class AdminClientBase:
         method: str,
         path: str,
         json: dict[str, Any] | None = None,
-        params: Mapping[str, str] | None = None,
+        params: Mapping[str, str] | Iterable[tuple[str, str]] | None = None,
     ) -> AbstractAsyncContextManager[aiohttp.ClientResponse]:
         pass
 
@@ -676,6 +881,17 @@ class AdminClientBase:
             cluster_name=payload["cluster_name"],
         )
 
+    def _parse_user_project_payload(
+        self, payload: dict[str, Any], user_name: str
+    ) -> ProjectUser:
+        return ProjectUser(
+            user_name=user_name,
+            role=ProjectUserRoleType(payload["role"]),
+            project_name=payload["project_name"],
+            cluster_name=payload["cluster_name"],
+            org_name=payload.get("org_name"),
+        )
+
     async def list_users(self) -> list[User]:
         async with self._request("GET", "users") as resp:
             resp.raise_for_status()
@@ -683,11 +899,67 @@ class AdminClientBase:
             users = [self._parse_user_payload(raw_user) for raw_user in users_raw]
         return users
 
+    @overload
     async def get_user(self, name: str) -> User:
-        async with self._request("GET", f"users/{name}") as resp:
+        ...
+
+    @overload
+    async def get_user(
+        self, name: str, *, include_clusters: Literal[True]
+    ) -> tuple[User, list[ClusterUser]]:
+        ...
+
+    @overload
+    async def get_user(
+        self, name: str, *, include_projects: Literal[True]
+    ) -> tuple[User, list[ProjectUser]]:
+        ...
+
+    @overload
+    async def get_user(
+        self,
+        name: str,
+        *,
+        include_clusters: Literal[True],
+        include_projects: Literal[True],
+    ) -> tuple[User, list[ClusterUser], list[ProjectUser]]:
+        ...
+
+    async def get_user(
+        self,
+        name: str,
+        *,
+        include_clusters: bool = False,
+        include_projects: bool = False,
+    ) -> GetUserRet:
+        params = []
+        if include_clusters:
+            params.append(("include", "clusters"))
+        if include_projects:
+            params.append(("include", "projects"))
+        async with self._request("GET", f"users/{name}", params=params) as resp:
             resp.raise_for_status()
-            raw_user = await resp.json()
-            return self._parse_user_payload(raw_user)
+            payload = await resp.json()
+            user = self._parse_user_payload(payload)
+            clusters: list[ClusterUser] | None = None
+            projects: list[ProjectUser] | None = None
+            if include_clusters:
+                clusters = [
+                    self._parse_user_cluster_payload(user_cluster_raw, user.name)
+                    for user_cluster_raw in payload["clusters"]
+                ]
+            if include_projects:
+                projects = [
+                    self._parse_user_project_payload(user_cluster_raw, user.name)
+                    for user_cluster_raw in payload["projects"]
+                ]
+            if clusters is not None and projects is not None:
+                return user, clusters, projects
+            if projects is not None:
+                return user, projects
+            if clusters is not None:
+                return user, clusters
+            return user
 
     async def get_user_with_clusters(self, name: str) -> tuple[User, list[ClusterUser]]:
         async with self._request(
@@ -708,6 +980,7 @@ class AdminClientBase:
         email: str,
         first_name: str | None = None,
         last_name: str | None = None,
+        skip_auto_add_to_clusters: bool = False,
     ) -> User:
         payload = {
             "name": name,
@@ -715,7 +988,14 @@ class AdminClientBase:
             "first_name": first_name,
             "last_name": last_name,
         }
-        async with self._request("POST", "users", json=payload) as resp:
+        async with self._request(
+            "POST",
+            "users",
+            json=payload,
+            params={
+                "skip_auto_add_to_clusters": _to_query_bool(skip_auto_add_to_clusters)
+            },
+        ) as resp:
             resp.raise_for_status()
             raw_user = await resp.json()
             return self._parse_user_payload(raw_user)
@@ -1606,11 +1886,19 @@ class AdminClientBase:
     async def create_org(
         self,
         name: str,
+        skip_auto_add_to_clusters: bool = False,
     ) -> Org:
         payload = {
             "name": name,
         }
-        async with self._request("POST", "orgs", json=payload) as resp:
+        async with self._request(
+            "POST",
+            "orgs",
+            json=payload,
+            params={
+                "skip_auto_add_to_clusters": _to_query_bool(skip_auto_add_to_clusters)
+            },
+        ) as resp:
             resp.raise_for_status()
             raw_org = await resp.json()
             return self._parse_org_payload(raw_org)
@@ -1764,6 +2052,322 @@ class AdminClientBase:
         ) as resp:
             resp.raise_for_status()
 
+    # projects
+
+    def _parse_project(self, payload: dict[str, Any]) -> Project:
+        return Project(
+            name=payload["name"],
+            cluster_name=payload["cluster_name"],
+            org_name=payload["org_name"],
+            is_default=payload["is_default"],
+            default_role=payload["default_role"],
+        )
+
+    async def create_project(
+        self,
+        name: str,
+        cluster_name: str,
+        org_name: str | None,
+        is_default: bool = False,
+        default_role: ProjectUserRoleType = ProjectUserRoleType.WRITER,
+    ) -> Project:
+        payload = {
+            "name": name,
+            "is_default": is_default,
+            "default_role": default_role,
+        }
+
+        if org_name:
+            url = f"clusters/{cluster_name}/orgs/{org_name}/projects"
+        else:
+            url = f"clusters/{cluster_name}/projects"
+
+        async with self._request(
+            "POST",
+            url,
+            json=payload,
+        ) as resp:
+            resp.raise_for_status()
+            return self._parse_project(await resp.json())
+
+    async def list_projects(
+        self, cluster_name: str, org_name: str | None = None
+    ) -> list[Project]:
+        if org_name:
+            url = f"clusters/{cluster_name}/orgs/{org_name}/projects"
+        else:
+            url = f"clusters/{cluster_name}/projects"
+
+        async with self._request(
+            "GET",
+            url,
+        ) as resp:
+            resp.raise_for_status()
+            return [self._parse_project(it) for it in await resp.json()]
+
+    async def get_project(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+    ) -> Project:
+        if org_name:
+            url = f"clusters/{cluster_name}/orgs/{org_name}/projects/{project_name}"
+        else:
+            url = f"clusters/{cluster_name}/projects/{project_name}"
+
+        async with self._request(
+            "GET",
+            url,
+        ) as resp:
+            resp.raise_for_status()
+            return self._parse_project(await resp.json())
+
+    async def update_project(self, project: Project) -> None:
+        payload = {
+            "is_default": project.is_default,
+            "default_role": project.default_role,
+        }
+
+        if project.org_name:
+            url = f"clusters/{project.cluster_name}/orgs/{project.org_name}/projects/{project.name}"
+        else:
+            url = f"clusters/{project.cluster_name}/projects{project.name}"
+
+        async with self._request(
+            "PUT",
+            url,
+            json=payload,
+        ) as resp:
+            resp.raise_for_status()
+
+    async def delete_project(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+    ) -> None:
+        if org_name:
+            url = f"clusters/{cluster_name}/orgs/{org_name}/projects/{project_name}"
+        else:
+            url = f"clusters/{cluster_name}/projects/{project_name}"
+
+        async with self._request(
+            "DELETE",
+            url,
+        ) as resp:
+            resp.raise_for_status()
+
+    # Project users
+
+    def _parse_project_user(
+        self, payload: dict[str, Any]
+    ) -> ProjectUser | ProjectUserWithInfo:
+        project_user = ProjectUser(
+            project_name=payload["project_name"],
+            cluster_name=payload["cluster_name"],
+            org_name=payload["org_name"],
+            user_name=payload["user_name"],
+            role=ProjectUserRoleType(payload["role"]),
+        )
+        if "user_info" in payload:
+            user_info = self._parse_user_info_payload(payload["user_info"])
+            project_user = project_user.add_info(user_info)
+        return project_user
+
+    @overload
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: Literal[True],
+    ) -> list[ProjectUserWithInfo]:
+        ...
+
+    @overload
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: Literal[False] = ...,
+    ) -> list[ProjectUser]:
+        ...
+
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: bool = False,
+    ) -> list[ProjectUser] | list[ProjectUserWithInfo]:
+        if org_name:
+            url = (
+                f"clusters/{cluster_name}/orgs/{org_name}/projects/{project_name}/users"
+            )
+        else:
+            url = f"clusters/{cluster_name}/projects/{project_name}/users"
+
+        async with self._request(
+            "GET",
+            url,
+            params={"with_user_info": _to_query_bool(with_user_info)},
+        ) as resp:
+            resp.raise_for_status()
+            return [self._parse_project_user(it) for it in await resp.json()]
+
+    @overload
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: Literal[True],
+    ) -> ProjectUserWithInfo:
+        ...
+
+    @overload
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: Literal[False] = ...,
+    ) -> ProjectUser:
+        ...
+
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: bool = False,
+    ) -> ProjectUser | ProjectUserWithInfo:
+        if org_name:
+            url = (
+                f"clusters/{cluster_name}/orgs/{org_name}"
+                f"/projects/{project_name}/users/{user_name}"
+            )
+        else:
+            url = f"clusters/{cluster_name}/projects/{project_name}/users/{user_name}"
+
+        async with self._request(
+            "GET",
+            url,
+            params={"with_user_info": _to_query_bool(with_user_info)},
+        ) as resp:
+            resp.raise_for_status()
+            return self._parse_project_user(await resp.json())
+
+    @overload
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: Literal[True],
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUserWithInfo:
+        ...
+
+    @overload
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: Literal[False] = ...,
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUser:
+        ...
+
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: bool = False,
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUser | ProjectUserWithInfo:
+        payload = {
+            "user_name": user_name,
+        }
+        if role:
+            payload["role"] = role.value
+
+        if org_name:
+            url = (
+                f"clusters/{cluster_name}/orgs/{org_name}/projects/{project_name}/users"
+            )
+        else:
+            url = f"clusters/{cluster_name}/projects/{project_name}/users"
+
+        async with self._request(
+            "POST",
+            url,
+            params={"with_user_info": _to_query_bool(with_user_info)},
+            json=payload,
+        ) as resp:
+            resp.raise_for_status()
+            return self._parse_project_user(await resp.json())
+
+    async def update_project_user(self, project_user: ProjectUser) -> None:
+        payload = {
+            "role": project_user.role.value,
+        }
+
+        if project_user.org_name:
+            url = (
+                f"clusters/{project_user.cluster_name}"
+                f"/orgs/{project_user.org_name}"
+                f"/projects/{project_user.project_name}"
+                f"/users/{project_user.user_name}"
+            )
+        else:
+            url = (
+                f"clusters/{project_user.cluster_name}"
+                f"/projects/{project_user.project_name}"
+                f"/users/{project_user.user_name}"
+            )
+
+        async with self._request(
+            "PUT",
+            url,
+            json=payload,
+        ) as resp:
+            resp.raise_for_status()
+
+    async def delete_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+    ) -> None:
+        if org_name:
+            url = (
+                f"clusters/{cluster_name}/orgs/{org_name}"
+                f"/projects/{project_name}/users/{user_name}"
+            )
+        else:
+            url = f"clusters/{cluster_name}/projects/{project_name}/users/{user_name}"
+
+        async with self._request(
+            "DELETE",
+            url,
+        ) as resp:
+            resp.raise_for_status()
+
     # OLD API:
 
     async def add_debt(
@@ -1903,6 +2507,22 @@ class AdminClientDummy(AdminClientABC):
         role=OrgUserRoleType.ADMIN,
         user_info=UserInfo(email="email@examle.com"),
     )
+    DUMMY_PROJECT = Project(
+        name="proj",
+        cluster_name="cluster",
+        org_name="org",
+        is_default=False,
+        default_role=ProjectUserRoleType.WRITER,
+    )
+
+    DUMMY_PROJECT_USER = ProjectUserWithInfo(
+        project_name="proj",
+        cluster_name="cluster",
+        org_name="org",
+        user_name="user",
+        role=ProjectUserRoleType.ADMIN,
+        user_info=UserInfo(email="email@examle.com"),
+    )
 
     async def __aenter__(self) -> "AdminClientDummy":
         return self
@@ -1916,8 +2536,46 @@ class AdminClientDummy(AdminClientABC):
     async def list_users(self) -> list[User]:
         return [self.DUMMY_USER]
 
+    @overload
     async def get_user(self, name: str) -> User:
-        return self.DUMMY_USER
+        ...
+
+    @overload
+    async def get_user(
+        self, name: str, *, include_clusters: Literal[True]
+    ) -> tuple[User, list[ClusterUser]]:
+        ...
+
+    @overload
+    async def get_user(
+        self, name: str, *, include_projects: Literal[True]
+    ) -> tuple[User, list[ProjectUser]]:
+        ...
+
+    @overload
+    async def get_user(
+        self,
+        name: str,
+        *,
+        include_clusters: Literal[True],
+        include_projects: Literal[True],
+    ) -> tuple[User, list[ClusterUser], list[ProjectUser]]:
+        ...
+
+    async def get_user(
+        self,
+        name: str,
+        *,
+        include_clusters: bool = False,
+        include_projects: bool = False,
+    ) -> GetUserRet:
+        if include_clusters is None and include_projects is None:
+            return self.DUMMY_USER
+        if include_projects is None:
+            return self.DUMMY_USER, [self.DUMMY_CLUSTER_USER]
+        if include_clusters is None:
+            return self.DUMMY_USER, [self.DUMMY_PROJECT_USER]
+        return self.DUMMY_USER, [self.DUMMY_CLUSTER_USER], [self.DUMMY_PROJECT_USER]
 
     async def get_user_with_clusters(self, name: str) -> tuple[User, list[ClusterUser]]:
         return self.DUMMY_USER, [self.DUMMY_CLUSTER_USER]
@@ -1928,6 +2586,7 @@ class AdminClientDummy(AdminClientABC):
         email: str,
         first_name: str | None = None,
         last_name: str | None = None,
+        skip_auto_add_to_clusters: bool = False,
     ) -> User:
         return self.DUMMY_USER
 
@@ -2361,6 +3020,7 @@ class AdminClientDummy(AdminClientABC):
     async def create_org(
         self,
         name: str,
+        skip_auto_add_to_clusters: bool = False,
     ) -> Org:
         return self.DUMMY_ORG
 
@@ -2452,6 +3112,155 @@ class AdminClientDummy(AdminClientABC):
     async def delete_org_user(self, org_name: str, user_name: str) -> None:
         pass
 
+    # projects
+
+    async def create_project(
+        self,
+        name: str,
+        cluster_name: str,
+        org_name: str | None,
+        is_default: bool = False,
+        default_role: ProjectUserRoleType = ProjectUserRoleType.WRITER,
+    ) -> Project:
+        return self.DUMMY_PROJECT
+
+    async def list_projects(
+        self, cluster_name: str, org_name: str | None = None
+    ) -> list[Project]:
+        return [self.DUMMY_PROJECT]
+
+    async def get_project(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+    ) -> Project:
+        return self.DUMMY_PROJECT
+
+    async def update_project(self, project: Project) -> None:
+        pass
+
+    async def delete_project(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+    ) -> None:
+        pass
+
+    #  project user
+
+    @overload
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: Literal[True],
+    ) -> list[ProjectUserWithInfo]:
+        ...
+
+    @overload
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: Literal[False] = ...,
+    ) -> list[ProjectUser]:
+        ...
+
+    async def list_project_users(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        with_user_info: bool = False,
+    ) -> list[ProjectUser] | list[ProjectUserWithInfo]:
+        return [self.DUMMY_PROJECT_USER]
+
+    @overload
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: Literal[True],
+    ) -> ProjectUserWithInfo:
+        ...
+
+    @overload
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: Literal[False] = ...,
+    ) -> ProjectUser:
+        ...
+
+    async def get_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        with_user_info: bool = False,
+    ) -> ProjectUser | ProjectUserWithInfo:
+        return self.DUMMY_PROJECT_USER
+
+    @overload
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: Literal[True],
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUserWithInfo:
+        ...
+
+    @overload
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: Literal[False] = ...,
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUser:
+        ...
+
+    async def create_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+        *,
+        with_user_info: bool = False,
+        role: ProjectUserRoleType | None = None,
+    ) -> ProjectUser | ProjectUserWithInfo:
+        return self.DUMMY_PROJECT_USER
+
+    async def update_project_user(self, project_user: ProjectUser) -> None:
+        pass
+
+    async def delete_project_user(
+        self,
+        project_name: str,
+        cluster_name: str,
+        org_name: str | None,
+        user_name: str,
+    ) -> None:
+        pass
+
     async def add_debt(
         self,
         cluster_name: str,
